# Scraper for Kansas Appeals Court (published)
# CourtID: kanctapp_p

from juriscraper.opinions.united_states.state import kan_p


class Site(kan_p.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.status = "Published"
        self.court = "Court of Appeals"



"""Scraper for Colorado Supreme Court
CourtID: colo
Court Short Name: Colo.
Author: Philip Ardery
Reviewer: mlr
Date created: 2016-06-03
Contact: Email "Internet and Technology" staff listed at http://www.cobar.org/staff
         they usually fix issues without responding to the emails directly. You can
         also try submitting the form here: http://www.cobar.org/contact
History:
    - 2022-01-31: Updated by William E. Palin
    - 2023-01-05: Updated by WEP
    - 2023-11-19: Drop Selenium by WEP
"""
import datetime
import re
from datetime import date, timedelta

from dateutil import parser

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.six_months_ago = date.today() - timedelta(180)
        self.status = "Published"
        self.url = f"https://www.courts.state.co.us/Courts/Supreme_Court/Proceedings/Index.cfm"

    def match_regex(self, str):
        """Match date regex patterns

        :param str: Date Str
        :return: Date object or none
        """
        date_match = re.search(
            r"\b(?:January|February|March|April|May|June|July|August|September|October|November|December)\s+\d{1,2},?\s+\d{4}\b|\b\d{1,2}/\d{1,2}/\d{2,4}\b",
            str,
        )
        if date_match:
            return parser.parse(date_match.group(0)).date()
        return None

    def extract_dates(self, row, partial_date):
        """Extract out one of the many date patterns

        :param row: Row to process
        :param partial_date: Partial date string
        :return: Date object and approximate boolean
        """
        raw_dates = row.xpath(
            "following-sibling::div/p/a/following-sibling::text()"
        )
        raw_date_str = " ".join(raw_dates)
        date = self.match_regex(raw_date_str)
        if date:
            return date, False
        raw_date_str = row.xpath("following-sibling::div/p/a/text()")[0]
        date = self.match_regex(raw_date_str)
        if date:
            return date, False
        date_object = datetime.datetime.strptime(partial_date, "%b %Y").date()
        return date_object, True

    def _process_html(self):
        for row in self.html.xpath("//div[@id='Dispositions']/*"):
            if row.tag == "a":
                docket, name = (
                    row.xpath("following-sibling::text()")[0]
                    .strip()
                    .split(" ", 1)
                )
                if "\xa0\xa0" not in name:
                    continue
                name, partial_date = name.split("\xa0\xa0")
                url = row.xpath("following-sibling::div/p/a/@href")[-1]
                date, date_filed_is_approximate = self.extract_dates(
                    row, partial_date
                )
                if date < self.six_months_ago:
                    continue
                self.cases.append(
                    {
                        "date": str(date),
                        "name": name,
                        "docket": docket,
                        "url": url,
                        "date_filed_is_approximate": date_filed_is_approximate,
                    }
                )



"""
Backscraper for Indiana Court of Appeals
CourtID: indctapp
Court Short Name: Ind. Ct. App.
Auth: Jon Andersen <janderse@gmail.com>
Reviewer: mlr
History:
    2014-09-02: Created by Jon Andersen
"""
from juriscraper.opinions.united_states_backscrapers.state import ind


class Site(ind.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://www.in.gov/judiciary/opinions/archapp.html"
        self.back_scrape_iterable = (
            "archapp2010.html",
            "archapp2008.html",
            "archapp2005.html",
        )

    def _download_backwards(self, page):
        self.url = f"http://www.in.gov/judiciary/opinions/{page}"
        self.html = self._download()



"""
Scraper for Oregon Court of Appeals
CourtID: orctapp
Court Short Name: OR Ct App
Author: William Palin
History:
    - 2023-11-18: Created
"""


from juriscraper.DeferringList import DeferringList
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            "https://www.courts.oregon.gov/publications/coa/Pages/default.aspx"
        )
        self.cases = []
        self.status = "Published"
        self.court_code = "p17027coll5"

    def fetch_url_json(self, identifier):
        """"""
        url = f"https://ojd.contentdm.oclc.org/digital/bl/dmwebservices/index.php?q=dmQuery/{self.court_code}/identi^{identifier}^all^and/title!subjec!descri!dmrecord/title/1024/1/0/0/0/0/json"
        json = self.request["session"].get(url).json()
        return f"https://ojd.contentdm.oclc.org/digital/api/collection/{self.court_code}/id/{json['records'][0]['pointer']}/download"

    def _process_html(self):
        for header in self.html.xpath("//h4//a/parent::h4"):
            date_string = header.text_content().strip()
            if not date_string:
                continue
            ul = header.xpath("./following-sibling::ul")[0]
            for item in ul.xpath(".//li"):
                # Ensure two links are present (skip Petitions for Review rows)
                # see or_example_2.html
                anchors = item.xpath(".//a")
                if not (len(anchors) > 1):
                    continue
                text = item.text_content().strip()
                url = anchors[0].xpath("./@href")[0]
                docket = anchors[1].text_content().strip()
                name = text.split(")", 1)[-1]
                self.cases.append(
                    {
                        "date": date_string,
                        "name": name,
                        "docket": docket,
                        "url": url,
                    }
                )

    def _get_download_urls(self):
        """Get download urls

        :return: List URLs
        """

        def fetcher(case):
            if self.test_mode_enabled():
                return case["url"]

            return self.fetch_url_json(case["url"].split("=")[-1][:-4])

        return DeferringList(seed=self.cases, fetcher=fetcher)
